@@ -34,21 +34,28 @@ class Operation:
         self.conn.close()
 
     def create_database(self, map: Mapping, all: str):
+        # Get charset and collation from mapping, or use defaults
+        charset = map.charset or 'utf8mb4'
+        collation = map.collation or 'utf8mb4_unicode_ci'
+
         if all == 'drop':
             logger.info(f'database {map.instance_name}')
             if database.is_exist(self.conn, map.instance_name):
                 database.drop(self.conn, map.instance_name)
-            database.create(self.conn, map.instance_name)
+            database.create(self.conn, map.instance_name, charset=charset, collation=collation)
         else:
             # 差分更新または個別指定の場合で、データベースが存在しない場合は作成する。
             if not database.is_exist(self.conn, map.instance_name):
                 logger.info(f'database {map.instance_name} was created.')
-                database.create(self.conn, map.instance_name)
+                database.create(self.conn, map.instance_name, charset=charset, collation=collation)
 
     def create_table(self, map: Mapping, schema: Schema, all: str, target: str, restore_only: bool = False):
         if restore_only:
             return
 
+        # テーブル再作成時に一緒に再作成したトリガーを記録
+        recreated_triggers = set()
+
         for tbl in schema.tables:
             if not all and target != tbl.table_name:
                 continue
@@ -65,6 +72,18 @@ class Operation:
             table.drop(self.conn, map.instance_name, tbl)
             table.create(self.conn, map.instance_name, tbl)
 
+            # テーブル再作成後、紐付くトリガーも一緒に再作成
+            for tr in schema.triggers:
+                if tr.table_name == tbl.table_name:
+                    logger.info(f'recreate trigger {map.instance_name}.{tr.trigger_name} (associated with table {tbl.table_name})')
+                    # トリガーが存在すれば削除
+                    if trigger.is_exist(self.conn, map.instance_name, tr):
+                        trigger.drop(self.conn, map.instance_name, tr)
+                    # トリガーを作成
+                    trigger.create(self.conn, map.instance_name, tr)
+                    # 再作成済みとして記録
+                    recreated_triggers.add(tr.trigger_name)
+
         for vw in schema.views:
             if not all and target != vw.view_name:
                 continue
@@ -79,6 +98,9 @@ class Operation:
                 view.create(self.conn, map.instance_name, vw)
 
         for tr in schema.triggers:
+            # テーブル再作成時に既に処理済みのトリガーはスキップ
+            if tr.trigger_name in recreated_triggers:
+                continue
             if not all and target != tr.trigger_name:
                 continue
             # トリガーが存在しない場合は作成する。
@@ -104,7 +126,7 @@ class Operation:
                 procedure.drop(self.conn, map.instance_name, proc)
                 procedure.create(self.conn, map.instance_name, proc)
 
-    def insert_data(self, map: Mapping, schema: Schema, all: bool, target: str, no_restore: bool = False, patch_file: str = None):
+    def insert_data(self, map: Mapping, schema: Schema, all: bool, target: str, no_restore: bool = False, patch_file: str = None, restore_backup: bool = False):
         if no_restore:
             # no_restore が指定されている場合は、初期データ投入もバックアップ復元もスキップ
             return
@@ -131,6 +153,9 @@ class Operation:
             # 個別指定時は従来通り
             datamodels_to_process = map.datamodels
 
+        # 処理済みテーブルを記録
+        processed_tables = set()
+
         # データ投入処理
         for dm in datamodels_to_process:
             if dm.sync_mode == const.SYNC_MODE_MANUAL and all:
@@ -140,6 +165,10 @@ class Operation:
             if not all and target != dm.table_name:
                 continue
             tbl = schema.tables[dm.table_name]
+
+            # 処理済みとしてマーク
+            processed_tables.add(dm.table_name)
+
             for ds in dm.datasources:
                 logger.info(f'insert {ds.filename} to {map.instance_name}.{tbl.table_name}')
                 ds.load()
@@ -157,6 +186,23 @@ class Operation:
 
             engine.commit(self.conn)
 
+        # datamodelがない場合でも、targetが指定されていてpatch/restore_backupが指定されていればリストア処理を実行
+        if target and target not in processed_tables and (patch_file or restore_backup):
+            if target not in schema.tables:
+                logger.warning(f'Table {target} not found in schema')
+                return
+            tbl = schema.tables[target]
+
+            if patch_file:
+                # パッチファイルが指定されている場合は、パッチを実行
+                self._execute_patch(map.instance_name, tbl.table_name, patch_file)
+            elif restore_backup and table.is_exist_backup(self.conn, map.instance_name, tbl, self.ymd):
+                # restore_backupが指定されている場合は、バックアップから復元
+                logger.info(f'restore {map.instance_name}.{tbl.table_name}')
+                table.restore(self.conn, map.instance_name, tbl, self.ymd)
+
+            engine.commit(self.conn)
+
     def _execute_patch(self, env: str, table_name: str, patch_file: str):
         """Execute patch file for data restoration."""
         from .patch import PatchConfig, generate_patch_sql, validate_patch_config
@@ -217,18 +263,51 @@ class Operation:
                     ds.load()
                     table.insert(self.conn, map.instance_name, tbl, ds.data)
 
+    def recreate_indexes_only(self, map: Mapping, schema: Schema, target: str):
+        """Recreate indexes for the specified table only."""
+        if target is None:
+            logger.error('target table must be specified for index-only mode')
+            return
+
+        # Find the target table in the schema
+        tbl = None
+        for t in schema.tables:
+            if t.table_name == target:
+                tbl = t
+                break
+
+        if tbl is None:
+            logger.error(f'table {target} not found in schema')
+            return
+
+        # Check if table exists
+        if not table.is_exist(self.conn, map.instance_name, tbl):
+            logger.error(f'table {map.instance_name}.{target} does not exist')
+            return
+
+        # Recreate indexes
+        logger.info(f'Recreating indexes for {map.instance_name}.{target}')
+        table.recreate_indexes(self.conn, map.instance_name, tbl)
+        engine.commit(self.conn)
+
 
 def apply(
         project, env: str, database: str, target: str, all: str, deploy: str,
-        no_restore: bool = False, restore_only: bool = False, patch: str = None, backup_key: str = None):
+        no_restore: bool = False, restore_only: bool = False, patch: str = None, backup_key: str = None,
+        index_only: bool = False, restore_backup: bool = False):
     """ データベースの適用処理を行う。 CLI向け関数. """
     with Operation(project, env, database, deploy, backup_key) as op:
         for map in op.environ.databases:
             if database is not None and map.instance_name != database:
                 continue
-            # データベースの作成
-            op.create_database(map, all)
 
             schema = map.build_schema(op.project.schemas, op.environ.schemas)
-            op.create_table(map, schema, all, target, restore_only)
-            op.insert_data(map, schema, all, target, no_restore, patch)
+
+            # index-only mode: recreate indexes only
+            if index_only:
+                op.recreate_indexes_only(map, schema, target)
+            else:
+                # Normal mode: create database and tables
+                op.create_database(map, all)
+                op.create_table(map, schema, all, target, restore_only)
+                op.insert_data(map, schema, all, target, no_restore, patch, restore_backup)
